/*
 * CrudContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.crud.core;

import dk.apaq.crud.Crud;
import dk.apaq.crud.CrudEvent;
import dk.apaq.crud.CrudListener;
import dk.apaq.crud.CrudNotifier;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * An abstract implementation of a <code>Crud</code> instance taking care
 * of CrudNotifer logic. An implementing class needs only to call the fire* methods(fx. fireOnRead)
 * and this class will make sure to fire evetns on the listeners.
 */
public abstract class BaseCrud<IDT, BT> implements
        Crud<IDT, BT>, CrudNotifier {

    private final List<WeakReference<CrudListener>> listeners = new ArrayList<WeakReference<CrudListener>>();

    /**
     * {@inheritDoc}
     */
    @Override
    public void addListener(CrudListener listener) {
        WeakReference<CrudListener> refListener = new WeakReference<CrudListener>(listener);
        listeners.add(refListener);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void removeListener(CrudListener listener) {
        synchronized(listeners) {
            Iterator<WeakReference<CrudListener>> it = listeners.iterator();
            while(it.hasNext()) {
                WeakReference<CrudListener> refListener = it.next();
                if(refListener.get() == listener) {
                    it.remove();
                    break;
                }
            }
        }
    }
    
    protected List<CrudListener> getListeners() {
        List<CrudListener> list = new ArrayList<CrudListener>();
        Iterator<WeakReference<CrudListener>> it = listeners.iterator();
        while(it.hasNext()) {
            WeakReference<CrudListener> refListener = it.next();
            if(refListener.get() == null) {
                it.remove();
               
            } else {
                list.add(refListener.get());
            }
        }
        return Collections.unmodifiableList(list);
    }

    public List<IDT> listIds() {
        return listIds(null);
    }

    public List<BT> list() {
        return list(null);
    }
    


    /**
     * Tells listeners that an entity has been read from crud.
     * @param entity The entity that was read.
     */
    protected void fireOnRead(IDT id, BT entity) {
        CrudEvent.WithEntity e = createEventWithEntity(id, entity);
        for (WeakReference<CrudListener> listener : listeners) {
            listener.get().onEntityRead(e);
        }
    }

    /**
     * Tells listeners that an entity is about tobe read.
     * @param id The id of the entity that is to be read.
     */
    protected void fireOnBeforeRead(IDT id) {
        CrudEvent.WithId e = createEventWithId(id);
        for (WeakReference<CrudListener> listener : listeners) {
            listener.get().onBeforeEntityRead(e);
        }
    }

    protected CrudEvent createEvent() {
        return new CrudEvent(this);
    }

    protected CrudEvent.WithId<IDT, BT> createEventWithId(IDT id) {
        return new CrudEvent.WithId<IDT, BT>(this, id);
    }

    protected CrudEvent.WithEntity<IDT, BT> createEventWithEntity(IDT id, BT entity) {
        return new CrudEvent.WithEntity<IDT, BT>(this, id, entity);
    }

}
